#include <p30f2011.h>

#define _kbUP		0b1
#define _kbDOWN		0b10
#define _kbLEFT		0b100
#define _kbRIGHT	0b1000
#define _kbA		0b10000
#define _kbB		0b100000
#define _kbSTART	0b10000000
#define _kbSELECT	0b1000000

unsigned char input;
unsigned char lastInput;
unsigned char lastChange;
unsigned char inA, inB, inC;

char release = 0;
char altCommand;

char rMess[10];

int NewPort;

char * message;
int messageI, messageLen;

void uartInit (void);
void uartSend (char[], int);

void timerInit (void);

void inputInit (void);

void makeAAP(char,char,char,char,char,int);

_FOSC(CSW_FSCM_OFF & FRC_PLL8);	//Set the internal FRC oscillator to run at 60MHz, so the dsPIC runs at 15MIPS

/************** END OF GLOBAL DEFINITIONS ***********/



/************* START OF MAIN FUNCTION ***************/

int main ( void )
{
	uartInit();
	char data[] = {0xFF,0x55,0x03,0x00,0x01,0x02,0xFA};	//Set iPod to Remote Mode
	uartSend(data,7);
	inputInit();
	timerInit();
	while(1)
	{
		Idle();
	}
}

void timerInit (void)
{
	PR1 = 0x0377;	//Reset the timer every 1/64 of a second
	T1CON = 0x8030;	//Timer 1 set on with a 1:256 prescaler
	_T1IP = 4;		//Generate a timer interrupt on each reset
	_T1IE = 1;
}

void inputInit (void)
{
	ADPCFG = 0xFF;
	TRISB = 0xFF;
	lastInput = 0xFF;
	lastChange = 0xFF;
}


void uartInit (void)
{
	U1BRG = 0x33;	//19200 baud

	_U1TXIP = 0;
	_U1RXIP = 0;
	IEC0bits.U1TXIE = 1;
	IEC0bits.U1RXIE = 1;

	U1STAbits.UTXISEL = 0;
	U1STAbits.UTXBRK = 0;
	U1STAbits.URXISEL = 0;
	U1STAbits.ADDEN = 0;

	U1MODE = 0x8400;
	U1STAbits.UTXEN = 1;
}

void uartSend (char cString[], int len)
{
	messageLen = len;
	message = cString;
	messageI = 0;
	_U1TXIF = 1;
	_U1TXIP = 0x5;
	Nop();
	while ((messageI < messageLen) || (!U1STAbits.TRMT))
	{
		Nop();
	}
	_U1TXIP = 0x0;
}

void makeAAP (char a,char b,char c, char d, char e, int args)
{
	int chk = 0x100;
	int i;
	rMess[0] = 0xFF;	//header
	rMess[1] = 0x55;	//header
	rMess[2] = args+1;	//Length
	rMess[3] = 0x02;	//Mode


	switch (args)
	{
		case 5:	rMess[8] = e;
		case 4:	rMess[7] = d;
		case 3:	rMess[6] = c;
		case 2:	rMess[5] = b;
				rMess[4] = a;
				break;
	}

	for (i=2; i<(args+4); i++)
		chk -= rMess[i];			//Generate a checksum
	rMess[args+4] = chk;

	uartSend(rMess, args+5);
}

/****** START OF INTERRUPT SERVICE ROUTINES *********/


void __attribute__((interrupt, no_auto_psv)) _U1TXInterrupt(void)
{
	while ((messageI < messageLen) && (!U1STAbits.UTXBF))
	{
		U1TXREG = *(message++);
		++messageI;
	}
	_U1TXIF = 0;
}

void __attribute__((interrupt, no_auto_psv)) _T1Interrupt(void)
{
	input = PORTB;

	altCommand = input & _kbSTART;
	input |= _kbSTART;

	if (input != lastInput)
		lastChange = lastInput;
	inA = input^lastChange;	
	inB = input^lastInput;
	inC = inA&(~input);		//Holds the most recent button that is being pressed now

	if (!release)
	{
		if (inB&input)
		{
			release = 1;
			makeAAP(0x00,0x00,0,0,0,2);		//Button released command
		} else if (altCommand)
		{
			if (inC)
			{
				if (inC&_kbUP)
					makeAAP(0x00,0x02,0,0,0,2);		//Volume Up
				else if (inC&_kbDOWN)
					makeAAP(0x00,0x04,0,0,0,2);		//Volume Down
				else if (inC&_kbLEFT)
					makeAAP(0x00,0x10,0,0,0,2);		//Previous Song
				else if (inC&_kbRIGHT)
					makeAAP(0x00,0x08,0,0,0,2);		//Next Song
				else if (inC&_kbA)
					makeAAP(0x00,0x01,0,0,0,2);		//Play
				else if (inC&_kbB)
					makeAAP(0x00,0x80,0,0,0,2);		//Stop
				else if (inC&_kbSELECT)
					makeAAP(0x00,0x00,0x04,0,0,3);	//Mute
			}
		} else
		{
			if (inC)
			{
				if (inC&_kbUP)
					makeAAP(0x00,0x00,0x00,0x00,0x01,5);	//Scroll Up
				else if (inC&_kbDOWN)
					makeAAP(0x00,0x00,0x00,0x00,0x02,5);	//Scroll Down
				else if (inC&_kbLEFT)
					makeAAP(0x00,0x00,0x00,0x40,0,4);		//Menu button
				else if (inC&_kbRIGHT)
					makeAAP(0x00,0x00,0x00,0x80,0,4);		//Select button
				else if (inC&_kbA)
					makeAAP(0x00,0x00,0x00,0x80,0,4);		//Select button
				else if (inC&_kbB)
					makeAAP(0x00,0x00,0x00,0x40,0,4);		//Menu Button
				else if (inC&_kbSELECT)
					makeAAP(0x00,0x80,0,0,0,2);				//Stop
			}
		}
	}
	if (input == 0xFF)	//No buttons pressed
		release = 0;
	lastInput = input;
	_T1IF = 0;
}

/********* END OF INTERRUPT SERVICE ROUTINES ********/
